summaryrefslogtreecommitdiff
path: root/lld
diff options
context:
space:
mode:
authorPeter Smith <peter.smith@arm.com>2023-03-10 13:22:18 +0000
committerPeter Smith <peter.smith@arm.com>2023-03-13 10:04:39 +0000
commit7ef47dd54e2d9f103986aec90aca554f634cc3e6 (patch)
tree5ce50e2955398b80f285bc04eeacff48c4137e45 /lld
parent40b273998af142aefd7e72657b0a76194f8de2db (diff)
downloadllvm-7ef47dd54e2d9f103986aec90aca554f634cc3e6.tar.gz
[LLD] Increase thunk pass limit
In issue 61250 https://github.com/llvm/llvm-project/issues/61250 there is an example of a program that takes 17 passes to converge, which is 2 more than the current limit of 15. Analysis of the program shows a particular section that is made up of many roughly thunk sized chunks of code ending in a call to a symbol that needs a thunk. Due to the positioning of the section, at each pass a subset of the calls go out of range of their original thunk, needing a new one created, which then pushes more thunks out of range. This process eventually stops after 17 passes. This patch is the simplest fix for the problem, which is to increase the pass limit. I've chosen to double it which should comfortably account for future cases like this, while only taking a few more seconds to reach the limit in case of non-convergence. As discussed in the issue, there could be some additional work done to limit thunk reuse, this would potentially increase the number of thunks in the program but should speed up convergence. Differential Revision: https://reviews.llvm.org/D145786
Diffstat (limited to 'lld')
-rw-r--r--lld/ELF/Writer.cpp4
1 files changed, 2 insertions, 2 deletions
diff --git a/lld/ELF/Writer.cpp b/lld/ELF/Writer.cpp
index e01e8ce2523b..534794bd835a 100644
--- a/lld/ELF/Writer.cpp
+++ b/lld/ELF/Writer.cpp
@@ -1647,8 +1647,8 @@ template <class ELFT> void Writer<ELFT>::finalizeAddressDependentContent() {
++pass;
// With Thunk Size much smaller than branch range we expect to
- // converge quickly; if we get to 15 something has gone wrong.
- if (changed && pass >= 15) {
+ // converge quickly; if we get to 30 something has gone wrong.
+ if (changed && pass >= 30) {
error(target->needsThunks ? "thunk creation not converged"
: "relaxation not converged");
break;